#region License
// Copyright (c) 2007 James Newton-King
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using NUnit.Framework;
using Utilities.Reflection;
using Utilities.Testing;
using Utilities.Miscellaneous;

namespace Utilities.Test.Reflection
{
  public class GenericOperatorFactoryFixture : TestFixtureBase
  {
    [Test]
    public void AddInt()
    {
      Func<int, int, int> add = GenericOperatorFactory<int, int, int, GenericOperatorFactoryFixture>.Add;

      Assert.AreEqual(2, add(1, 1));
      Assert.AreEqual(3, add(1, 2));
      Assert.AreEqual(1, add(1, 0));
      Assert.AreEqual(0, add(1, -1));
      Assert.AreEqual(int.MaxValue - 1, add(int.MaxValue, -1));
    }

    [Test]
    public void AddDouble()
    {
      Func<double, double, double> add = GenericOperatorFactory<double, double, double, GenericOperatorFactoryFixture>.Add;

      Assert.AreEqual(2.5, add(1.0, 1.5));
      Assert.AreEqual(3.0, add(1.0, 2.0));
      Assert.AreEqual(double.NaN, add(1, double.NaN));
    }

    [Test]
    public void AddDateTime()
    {
      Func<double, double, double> add = GenericOperatorFactory<double, double, double, GenericOperatorFactoryFixture>.Add;

      Assert.AreEqual(2.5, add(1.0, 1.5));
      Assert.AreEqual(3.0, add(1.0, 2.0));
      Assert.AreEqual(double.NaN, add(1, double.NaN));
    }

    [Test]
    public void MultiplyInt()
    {
      Func<int, int, int> multiply = GenericOperatorFactory<int, int, int, GenericOperatorFactoryFixture>.Multiply;

      Assert.AreEqual(1, multiply(1, 1));
      Assert.AreEqual(4, multiply(2, 2));
    }

    [Test]
    public void DevideInt()
    {
      Func<int, int, int> devide = GenericOperatorFactory<int, int, int, GenericOperatorFactoryFixture>.Divide;

      Assert.AreEqual(1, devide(1, 1));
      Assert.AreEqual(2, devide(4, 2));
      Assert.AreEqual(3, devide(9, 3));
      Assert.AreEqual(2, devide(20, 10));
    }
  }
}