using System;
using System.Collections.Generic;
using System.Text;
using Utilities.Testing;
using NUnit.Framework;
using Utilities.Text;

namespace Utilities.Test.Text
{
  public class StringBufferFixture : TestFixtureBase
  {
    [Test]
    public void ConstructorEmpty()
    {
      StringBuffer buffer = new StringBuffer();
      Assert.AreEqual(string.Empty, buffer.ToString());
      Assert.AreEqual(0, buffer.Position);
    }

    [Test]
    public void ConstructorInitialSize()
    {
      StringBuffer buffer = new StringBuffer(10000);
      Assert.AreEqual(string.Empty, buffer.ToString());
      Assert.AreEqual(0, buffer.Position);

      buffer.Append('!');
      Assert.AreEqual("!", buffer.ToString());
      Assert.AreEqual(1, buffer.Position);
    }

    [Test]
    [ExpectedException(typeof(ArgumentOutOfRangeException), ExpectedMessage = @"Argument cannot be negative.
Parameter name: initialSize
Actual value was -80.")]
    public void ConstructorNegativeInitialSize()
    {
      new StringBuffer(-80);
    }

    [Test]
    public void ConstructorInitialBuffer()
    {
      string testString = "This is a test string.";

      StringBuffer buffer = new StringBuffer(testString.ToCharArray());
      Assert.AreEqual(testString, buffer.ToString());
      Assert.AreEqual(testString.Length, buffer.Position);

      buffer.Append('!');
      Assert.AreEqual(testString + "!", buffer.ToString());
      Assert.AreEqual(testString.Length + 1, buffer.Position);
    }

    [Test]
    [ExpectedException(typeof(ArgumentNullException), ExpectedMessage = @"Value cannot be null.
Parameter name: internalBuffer")]
    public void ConstructorInitialBufferNull()
    {
      new StringBuffer(null);
    }

    [Test]
    public void Append()
    {
      StringBuffer buffer = new StringBuffer();
      buffer.Append('0');
      buffer.Append('1');
      buffer.Append('2');
      buffer.Append('3');
      buffer.Append('4');

      Assert.AreEqual("01234", buffer.ToString());
    }

    [Test]
    public void Clear()
    {
      StringBuffer buffer = new StringBuffer("Pie".ToCharArray());
      Assert.AreEqual("Pie", buffer.ToString());
      Assert.AreEqual(3, buffer.Position);

      buffer.Clear();
      Assert.AreEqual(string.Empty, buffer.ToString());
      Assert.AreEqual(0, buffer.Position);
    }

    [Test]
    [ExpectedException(typeof(ArgumentOutOfRangeException), ExpectedMessage = @"Argument cannot be negative.
Parameter name: value
Actual value was -1.")]
    public void PositionNegative()
    {
      StringBuffer buffer = new StringBuffer();
      buffer.Position = -1;
    }

    [Test]
    public void PositionOutsideBounds()
    {
      StringBuffer buffer = new StringBuffer();
      buffer.Position = 10;
      buffer.Append('!');

      Assert.AreEqual("\0\0\0\0\0\0\0\0\0\0\0!", buffer.ToString());
      Assert.AreEqual(11, buffer.Position);
    }

    [Test]
    public void PositionChangeAppend()
    {
      StringBuffer buffer = new StringBuffer();
      buffer.Append('0');
      buffer.Append('1');
      buffer.Append('2');
      buffer.Append('3');
      buffer.Append('4');

      Assert.AreEqual("01234", buffer.ToString());
      Assert.AreEqual(5, buffer.Position);

      buffer.Position = 3;
      buffer.Append('5');
      buffer.Append('6');

      Assert.AreEqual("01256", buffer.ToString());
      Assert.AreEqual(5, buffer.Position);
    }

    [Test]
    public void AppendArray()
    {
      StringBuffer buffer = new StringBuffer();
      buffer.Append('0');
      buffer.Append('1');
      buffer.Append('2');
      buffer.Append('3');
      buffer.Append('4');

      Assert.AreEqual("01234", buffer.ToString());
      Assert.AreEqual(5, buffer.Position);

      buffer.Append("012345678901234567890123456789".ToCharArray());
      Assert.AreEqual("01234012345678901234567890123456789", buffer.ToString());
      Assert.AreEqual(35, buffer.Position);

      buffer.Append('!');
      Assert.AreEqual("01234012345678901234567890123456789!", buffer.ToString());
      Assert.AreEqual(36, buffer.Position);
    }
  }
}